package com.yunzhi.nanyang.common;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import org.springframework.core.io.ClassPathResource;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.function.BiConsumer;

public class ExcelUtils {

    /**
     * 发送 excel 到客户端
     * 暂时只支持单 sheet 页
     * @param response
     * @param data
     * @param fileName
     * @throws IOException
     */
    public static void flush(HttpServletResponse response, Class dataClass, List data, String fileName) throws IOException {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename="+StringUtils.urlEncode(fileName)+".xlsx");

        EasyExcel.write(response.getOutputStream(), dataClass).sheet("sheet1").doWrite(data);
    }

    /**
     * 模板导出
     * @param response
     * @param tpl
     * @param data
     * @param fileName
     * @param callback
     * @throws IOException
     */
    public static void tplExport(HttpServletResponse response, String tpl, List data, String fileName, BiConsumer<ExcelWriter, WriteSheet> callback) throws IOException {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename="+StringUtils.urlEncode(fileName)+".xlsx");

        ClassPathResource tplResource = new ClassPathResource(tpl);
        InputStream tplResourceInputStream = tplResource.getInputStream();

        ExcelWriter excelWriter = EasyExcel.write(response.getOutputStream()).withTemplate(tplResourceInputStream).build();
        WriteSheet writeSheet = EasyExcel.writerSheet().build();
        // 这里注意 入参用了forceNewRow 代表在写入list的时候不管list下面有没有空行 都会创建一行，然后下面的数据往后移动。默认 是false，会直接使用下一行，如果没有则创建。
        // forceNewRow 如果设置了true,有个缺点 就是他会把所有的数据都放到内存了，所以慎用
        // 简单的说 如果你的模板有list,且list不是最后一行，下面还有数据需要填充 就必须设置 forceNewRow=true 但是这个就会把所有数据放到内存 会很耗内存
        // 如果数据量大 list不是最后一行 参照下一个
        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();
        excelWriter.fill(data, fillConfig, writeSheet);

        // 回填部分数据
        callback.accept(excelWriter, writeSheet);

        excelWriter.finish();
    }
}
