package com.yunzhi.nanyang.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.ExcelUtils;
import com.yunzhi.nanyang.common.ResponseBean;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.entity.TaMachinery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.nanyang.entity.TaDeviceJob;
import com.yunzhi.nanyang.service.TaDeviceJobService;

import javax.servlet.http.HttpServletResponse;

/**
 * 设备作业表;(ta_device_job)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-11-3
 */
@Api(tags = "设备作业表对象功能接口")
@RestController
@RequestMapping("/admin")
public class TaDeviceJobController extends BaseController {

    @Autowired
    private TaDeviceJobService taDeviceJobService;

    /**
     * 通过ID查询单条数据
     *
     * @param userId 用户ID
     * @return 实例对象
     */
    @ApiOperation("获取用户作业农机列表")
    @GetMapping("/device-job/user/{userId}/machinery")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String userId) throws Exception {
        List<TaMachinery> machineryList = taDeviceJobService.getMachineryListByUser(userId);
        return ResponseBean.success(machineryList);
    }

    /**
     * 设备作业列表
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("设备作业列表")
    @GetMapping("/device-job")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("设备分类") @RequestParam(value = "deviceKind", required = false) String deviceKind,
                             @ApiParam("设备编号") @RequestParam(value = "deviceNo", required = false) String deviceNo,
                             @ApiParam(value = "开始时间", example = "2022-10-01") @RequestParam(value = "startDate", required = false) String startDate,
                             @ApiParam(value = "结束时间", example = "2022-10-31") @RequestParam(value = "endDate", required = false) String endDate) throws Exception {

        IPage<TaDeviceJob> pg = new Page<>(pageNum, pageSize);
        IPage<TaDeviceJob> result = taDeviceJobService.getPageBy(pg, deviceKind, deviceNo, startDate, endDate);

        return ResponseBean.success(result);
    }


    /**
     * 设备作业统计
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("设备作业统计")
    @GetMapping("/device-job/by-org")
    public ResponseBean statis(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                               @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                               @ApiParam("合作社ID") @RequestParam(value = "orgId", required = false) String orgId,
                               @ApiParam("合作社名称") @RequestParam(value = "orgName", required = false) String orgName,
                               @ApiParam("农机名称") @RequestParam(value = "machineryName", required = false) String machineryName,
                               @ApiParam("人员名称") @RequestParam(value = "userName", required = false) String userName,
                               @ApiParam("农机ID") @RequestParam(value = "machineryId", required = false) String machineryId,
                               @ApiParam("人员ID") @RequestParam(value = "userId", required = false) String userId,
                               @ApiParam("人员手机") @RequestParam(value = "phone", required = false) String phone,
                               @ApiParam("乡镇") @RequestParam(value = "town", required = false) String town,
                               @ApiParam("设备分类") @RequestParam(value = "deviceKind", required = false) String deviceKind,
                               @ApiParam("设备编号") @RequestParam(value = "deviceNo", required = false) String deviceNo,
                               @ApiParam(value = "开始时间", example = "2022-10-01") @RequestParam(value = "startDate") String startDate,
                               @ApiParam(value = "结束时间", example = "2022-10-31") @RequestParam(value = "endDate") String endDate) throws Exception {

        IPage<TaDeviceJob> pg = new Page<>(pageNum, pageSize);

        Map<String, Object> params = new HashMap<>();
        params.put("orgId", orgId);
        params.put("orgName", orgName);
        params.put("deviceKind", deviceKind);
        params.put("deviceNo", deviceNo);
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("machineryName", machineryName);
        params.put("userName", userName);
        params.put("phone", phone);
        params.put("machineryId", machineryId);
        params.put("userId", userId);
        params.put("town", town);

        IPage<TaDeviceJob> list = taDeviceJobService.statisOrgBy(pg, params);
        Double totalArea = taDeviceJobService.statisTotalArea(params);
        return ResponseBean.success(new HashMap<String, Object>(){{
            put("list", list);
            put("totalArea", totalArea);
        }});
    }

    /**
     * 设备作业统计-导出
     * @param orgName
     * @param deviceKind
     * @param deviceNo
     * @param startDate
     * @param endDate
     * @param response
     * @return
     * @throws Exception
     */
    @ApiOperation("设备作业统计-导出")
    @GetMapping("/device-job/by-org/export")
    public ResponseBean export(@ApiParam("合作社名称") @RequestParam(value = "orgName", required = false) String orgName,
                               @ApiParam("合作社ID") @RequestParam(value = "orgId", required = false) String orgId,
                               @ApiParam("农机名称") @RequestParam(value = "machineryName", required = false) String machineryName,
                               @ApiParam("人员名称") @RequestParam(value = "userName", required = false) String userName,
                               @ApiParam("农机ID") @RequestParam(value = "machineryId", required = false) String machineryId,
                               @ApiParam("人员ID") @RequestParam(value = "userId", required = false) String userId,
                               @ApiParam("人员手机") @RequestParam(value = "phone", required = false) String phone,
                               @ApiParam("乡镇") @RequestParam(value = "town", required = false) String town,
                               @ApiParam("设备分类") @RequestParam(value = "deviceKind", required = false) String deviceKind,
                               @ApiParam("设备编号") @RequestParam(value = "deviceNo", required = false) String deviceNo,
//                               @ApiParam("位置") @RequestParam(value = "address", required = false) String address,
                               @ApiParam(value = "开始时间", example = "2022-10-01") @RequestParam(value = "startDate") String startDate,
                               @ApiParam(value = "结束时间", example = "2022-10-31") @RequestParam(value = "endDate") String endDate,
                               HttpServletResponse response) throws Exception {

        Map<String, Object> params = new HashMap<>();
        params.put("orgId", orgId);
        params.put("orgName", orgName);
        params.put("deviceKind", deviceKind);
        params.put("deviceNo", deviceNo);
        params.put("startDate", startDate);
        params.put("endDate", endDate);
        params.put("machineryName", machineryName);
        params.put("userName", userName);
        params.put("phone", phone);
        params.put("machineryId", machineryId);
        params.put("userId", userId);
        params.put("town", town);

        List<TaDeviceJob> list = taDeviceJobService.listOrgBy(params);

        Double total = 0.0;
        if (list != null || list.size() > 0) {
            for (TaDeviceJob item : list) {
                if ("shensong".equals(item.getDeviceKind())) {
                    item.setDeviceKind("深松机");
                } else {
                    item.setDeviceKind("无人机");
                }

                if (null != item.getJobArea()) {
                    total += item.getJobArea();
                }
            }
        }

        try {
            String date = String.format("%s 至 %s", startDate, endDate);
            Double finalTotal = total;
            ExcelUtils.tplExport(response, "job.xlsx", list, "作业统计", (excelWriter, writeSheet) -> {
                Map<String, Object> map = new HashMap<>();
                map.put("date", date);
                map.put("total", finalTotal);
                excelWriter.fill(map, writeSheet);
            });

            return null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResponseBean.error("导出失败, 请重试");
    }

//    /**
//     * 新增数据
//     *
//     * @param taDeviceJob 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("新增数据")
//    @PostMapping("/device-job")
//    public ResponseBean add(@ApiParam("对象实体") @RequestBody TaDeviceJob taDeviceJob) throws Exception {
//        taDeviceJobService.save(taDeviceJob);
//        return ResponseBean.success(taDeviceJob);
//    }
//
//    /**
//     * 更新数据
//     *
//     * @param taDeviceJob 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("更新数据")
//    @PutMapping("/device-job/{id}")
//    public ResponseBean edit(@ApiParam("对象实体") @RequestBody TaDeviceJob taDeviceJob,
//                             @ApiParam("对象ID") @PathVariable String id) throws Exception {
//        taDeviceJobService.updateById(taDeviceJob);
//        return ResponseBean.success(taDeviceJob);
//    }
//
//    /**
//     * 通过主键删除数据
//     *
//     * @param jobId 主键
//     * @return 是否成功
//     */
//    @ApiOperation("通过主键删除数据")
//    @DeleteMapping("/device-job/{id}")
//    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
//        taDeviceJobService.removeLogicById(id);
//        return ResponseBean.success("success");
//    }
}