package com.yunzhi.nanyang.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.BaseController;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.ResponseBean;
import com.yunzhi.nanyang.entity.TaWorkJob;
import com.yunzhi.nanyang.entity.TdDevice;
import com.yunzhi.nanyang.entity.TdHatcDevice;
import com.yunzhi.nanyang.service.ITaMachineryService;
import com.yunzhi.nanyang.service.ITaWorkJobService;
import com.yunzhi.nanyang.service.ITdDeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.nanyang.service.ITaMachineryGpsService;
import com.yunzhi.nanyang.entity.TaMachineryGps;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
    * 农机定位信息 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2022-03-08
 */

@Api(tags = "农机定位信息")
@RestController
@RequestMapping("/")
public class TaMachineryGpsController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaMachineryGpsController.class);

    @Autowired
    public ITaMachineryGpsService iTaMachineryGpsService;

    @Autowired
    public ITaWorkJobService iTaWorkJobService;

    @Autowired
    public ITdDeviceService iTdDeviceService;


    /**
     * 分页查询列表
     * @return
     */
    @RequestMapping(value="/admin/machinery-gps",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taMachineryGpsList(@ApiParam("作业ID") @RequestParam(value ="jobId") String jobId) throws Exception {

        TaWorkJob taWorkJob = iTaWorkJobService.getById(jobId);
        if (null == taWorkJob || taWorkJob.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("未找到农机作业信息");
        }

        if (taWorkJob.getStatus() < Constants.WORK_DOING) {
            return ResponseBean.error("农机尚未作业");
        }

        if (null == taWorkJob.getStartDate() || null == taWorkJob.getEndDate()) {
            return ResponseBean.error("未找到农机作业时间");
        }

        // 获取设备信息
        TdDevice tdDevice = iTdDeviceService.getExistBy("machinery_id", taWorkJob.getMachineryId(), false, true);
        if (null == tdDevice) {
            return ResponseBean.error("未找到作业农机的GPS设备信息");
        }

        // apiconfig 字符串格式一般为
        // {"channel":1,"netType":1,"deviceId":"52bb072175e914ffae1db96cff652817"}
        JSONObject jsonObject = JSON.parseObject(tdDevice.getApiConfig());
        String deviceId = jsonObject.getString("deviceId");

        QueryWrapper<TaMachineryGps> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_id", deviceId);
        queryWrapper.ge("create_date", taWorkJob.getStartDate());
        queryWrapper.le("create_date", taWorkJob.getEndDate());
        queryWrapper.orderByAsc("create_date");

        List<TaMachineryGps> result = iTaMachineryGpsService.list(queryWrapper);
        return ResponseBean.success(result);
    }

//    /**
//     * 保存对象
//     * @param taMachineryGps 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taMachineryGps",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taMachineryGpsAdd(@ApiParam("保存内容") @RequestBody TaMachineryGps taMachineryGps) throws Exception{
//
//        if (iTaMachineryGpsService.save(taMachineryGps)){
//            return ResponseBean.success(taMachineryGps);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taMachineryGps/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taMachineryGpsDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaMachineryGpsService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taMachineryGps 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taMachineryGps/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taMachineryGpsUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaMachineryGps taMachineryGps) throws Exception{
//
//        if (iTaMachineryGpsService.updateById(taMachineryGps)){
//            return ResponseBean.success(iTaMachineryGpsService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taMachineryGps/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taMachineryGpsGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaMachineryGpsService.getById(id));
//    }
}
