package com.yunzhi.nanyang.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.nanyang.common.*;

import java.time.LocalDateTime;
import java.util.List;

import com.yunzhi.nanyang.entity.TcBind;
import com.yunzhi.nanyang.entity.TdDevice;
import com.yunzhi.nanyang.service.ITdDeviceService;
import com.yunzhi.nanyang.service.TcBindService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.nanyang.entity.TdRawDevice;
import com.yunzhi.nanyang.service.TdRawDeviceService;

/**
 * 原始设备信息;(td_raw_device)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-11-3
 */
@Api(tags = "原始设备信息对象功能接口")
@RestController
@RequestMapping("/")
public class TdRawDeviceController extends BaseController {

    @Value("${device.server}")
    String deviceServer;

    @Autowired
    HttpUtils httpUtils;

    @Autowired
    private TdRawDeviceService tdRawDeviceService;

    @Autowired
    private ITdDeviceService iTdDeviceService;

    @Autowired
    private TcBindService tcBindService;

//    /** 
//     * 通过ID查询单条数据 
//     *
//     * @param deviceId 主键
//     * @return 实例对象
//     */
//    @ApiOperation("通过ID查询单条数据")
//    @GetMapping("/raw-device/{id}")
//    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
//        return ResponseBean.success(tdRawDeviceService.getById(id));
//    }

//    /**
//     * 分页查询
//     *
//     * @param pageNum  当前页码
//     * @param pageSize 每页条数
//     * @return 查询结果
//     */
//    @ApiOperation("分页查询")
//    @GetMapping("/admin/raw-device")
//    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
//                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
//                             @ApiParam("设备分类") @RequestParam(value = "deviceKind", required = false) String deviceKind,
//                             @ApiParam("设备状态") @RequestParam(value = "deviceStatus", required = false) Integer deviceStatus) throws Exception {
//
//        IPage<TdRawDevice> pg = new Page<>(pageNum, pageSize);
//        QueryWrapper<TdRawDevice> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq(!StringUtils.isEmpty(deviceKind), "device_kind", deviceKind);
//        queryWrapper.eq(null != deviceStatus, "device_status", deviceStatus);
//        queryWrapper.orderByDesc("create_date");
//        IPage<TdRawDevice> result = tdRawDeviceService.page(pg);
//
//        return ResponseBean.success(result);
//    }

    /**
     * 新增数据
     *
     * @param tdRawDevice 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增设备")
    @PostMapping("/admin/raw-device")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody TdRawDevice tdRawDevice) throws Exception {

        if (StringUtils.isEmpty(tdRawDevice.getDeviceKind())) {
            return ResponseBean.error("设备大类不能为空");
        }

        if (StringUtils.isEmpty(tdRawDevice.getDeviceNo())) {
            return ResponseBean.error("设备号不能为空");
        }

        QueryWrapper<TdRawDevice> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("device_kind", tdRawDevice.getDeviceKind());
        queryWrapper.eq("device_no", tdRawDevice.getDeviceNo());

        TdRawDevice origin = tdRawDeviceService.getOne(queryWrapper);
        if (null != origin) {
            return ResponseBean.error("当前设备号已存在");
        }

        TcBind tcBind = null;
        String url = String.format("%s/%s/tdRawDevice/%s", deviceServer, tdRawDevice.getDeviceKind(), tdRawDevice.getDeviceNo());
        String s = httpUtils.get(url);
        if (StringUtils.isEmpty(s)) {
            return ResponseBean.error("验证平台服务出错, 请联系技术人员");
        } else {
            JSONObject resp = JSONObject.parseObject(s);
            if (!"1000".equals(resp.getString("code"))) {
                String msg = resp.getString("message");
                if (!StringUtils.isEmpty(msg)) {
                    return ResponseBean.error(msg);
                } else {
                    return ResponseBean.error("验证平台服务出错, 未找到当前设备");
                }
            }

            // 飞防设备在新增的时候，把中间数据采集表也写入
            if ("feifang".equals(tdRawDevice.getDeviceKind())) {
                JSONObject data = resp.getJSONObject("data");
                if (data != null) {
                    tcBind = tcBindService.getByDevice(tdRawDevice.getDeviceKind(), tdRawDevice.getDeviceNo());
                    if (null == tcBind) {
                        tcBind = new TcBind();
                        tcBind.setDeviceType(tdRawDevice.getDeviceKind());
                        tcBind.setDeviceNo(tdRawDevice.getDeviceNo());
                        tcBind.setMachineryType(data.getString("p1"));
                        tcBind.setMachineryName(data.getString("p25"));
                        tcBind.setUserName(data.getString("p8"));
                        tcBind.setPhone(data.getString("p7"));
                        tcBind.setStatus(0);
                        tcBind.setCreateDate(LocalDateTime.now());
                    }
                }

            }
        }

        tdRawDevice.setDeviceStatus(0); // 默认离线
        tdRawDevice.setCreateDate(LocalDateTime.now());
        tdRawDeviceService.save(tdRawDevice);

        if (tcBind != null && null == tcBind.getSerialNo()) {
            tcBindService.save(tcBind);
        }

        return ResponseBean.success(tdRawDevice);
    }

    /**
     * 同步设备
     *
     * @param
     * @return 实例对象
     */
    @ApiOperation("同步设备")
    @PutMapping("/admin/raw-device/sync")
    public ResponseBean sync() throws Exception {
        String url = deviceServer + "/shensong/syncDevice";
        String s = httpUtils.get(url);
        return JSONObject.parseObject(s, ResponseBean.class);
    }

    /**
     * 同步设备定位信息
     *
     * @param
     * @return 实例对象
     */
    @ApiOperation("同步设备定位信息")
    @PutMapping("/admin/raw-device/sync-loc")
    public ResponseBean syncLoc(@ApiParam("设备类型") @RequestParam String deviceType,
                                @ApiParam("设备号") @RequestParam String deviceNo) throws Exception {
        String url = String.format("%s/%s/sync-loc/%s", deviceServer, deviceType, deviceNo);
        String s = httpUtils.get(url);
        return JSONObject.parseObject(s, ResponseBean.class);
    }

    /**
     * 通过主键删除数据
     *
     * @param deviceId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/admin/raw-device/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id){

        TdRawDevice tdRawDevice = tdRawDeviceService.getById(id);
        if (null == tdRawDevice) {
            return ResponseBean.error("未找到数据");
        }

        TdDevice tdDevice = iTdDeviceService.getExistByDevice(tdRawDevice.getDeviceKind(), tdRawDevice.getDeviceNo());
        if (null != tdDevice) {
            tdDevice.setStatus(Constants.STATUS_DELETE);
            iTdDeviceService.updateById(tdDevice);
        }

        tdRawDeviceService.removeById(id);
        return ResponseBean.success("success");
    }
}